//*******************************************************************************
//* Dieses Programm wurde von Ulrich Radig fr Atmel AVR C geschrieben 	 	*
//* 																			*
//* Ich wnsche jeden viel Spa mit diesen kleinen Web-Server der ohne 			*
//* zustzlichen Speicher auskommt.												*
//*																				*
//* Bei Fragen und Verbesserungen wendet euch per EMail an						*
//*																				*
//* mail@ulrichradig.de															*
//*																				*
//* oder im Forum meiner Web Page : www.ulrichradig.de							*
//*																				*
//*******************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
//#include <avr/iom128.h> wird im Makefile definiert

//Anpassen der Register wenn ein ATMega128 benutzt wird
#if defined (__AVR_ATmega128__)
#	define USR UCSR0A
#	define UCR UCSR0B
#	define UDR UDR0
#	define UBRR UBRR0L
#	define EICR EICRB
#endif

#if defined (__AVR_ATmega32__)
#	define USR UCSRA
#	define UCR UCSRB
#	define UBRR UBRRL
#	define EICR EICRB
#endif

//Use the FTP Demon then set USE_FTP 1
#define USE_FTPD	1

//Use the Clock then set USE_CLOCK 1
#define USE_CLOCK	1

//Der Quarz auf dem Experimentierboard hat eine Frequenz von 14.318MHz
#define sysclk	14318180	//Quarz Frequenz in Hz

//Die Baud_Rate der Seriellen Schnittstelle ist 9600 Baud
#define baud_rate 9600		//Baud Rate fr die Serielle Schnittstelle	

//MTU Size setzen
#define MTU_SIZE  1400

//MAC Addresse des Wep Servers
const unsigned char mymac[] = {30,10,43,43,45,77};

//IP Addresse des Wep Servers
const unsigned char MYIP[] = {192,168,0,101};

unsigned long Seq_counter;
unsigned long Ack_counter;

#if USE_CLOCK
	//Include Datei zur Verarbeitung von TCP (ftpd) Packete
	#include <clock.c>
#endif

//Include Datei zum einrichten des NIC's sowie
//Routinen zum Daten Empfangen / Senden (DMA READ / WRITE)
#include <device.c>

//Include Datei zur Berechnung der Checksumme
#include <checksum.c>

//Include Datei zur Verarbeitung des IP - Header's
#include <ip.c>

//Include Datei zur Verarbeitung von ARP Packete Auflsung von 
//IP-Addresse zur MAC-Adresse (IP over Ethernet)
#include <arp.c>

//Include Datei zur Verarbeitung von ICMP Packete (PING)
#include <icmp.c>

//Include Datei zur Verarbeitung von TCP Packete
#include <tcp.c>

//Include Datei zur Verarbeitung von TCP (httpd) Packete
#include <httpd.c>

#if USE_FTPD
	//Include Datei zur Verarbeitung von TCP (ftpd) Packete
	#include <ftpd.c>
#endif

//Include Datei zur Verarbeitung von TCP (telnet) Packete
#include <telnet.c>

//Include Datei zur Abarbeitung des Interrupt wenn Daten empfangen werden
#include <interrupt4.c>

//INstallation der Seriellen Schnittstelle
void IOInit (void)
{
	//Enable TXEN im Register UCR TX-Data Enable
	outp ((1 << TXEN),UCR);
	//Teiler wird gesetzt 
	outp ((sysclk / (baud_rate * 16L) - 1) , UBRR);
}


//Routine fr printf
int uart_putchar (char c)
{
	if (c == '\n')
		uart_putchar('\r');
	//Warten solange bis Zeichen gesendet wurde
	loop_until_bit_is_set(USR, UDRE);
	UDR = c;
	return (0);
}

//Hauptprogramm
int main (void)
{
	IOInit();
	//ffnet einen kanal fr printf
	fdevopen (uart_putchar, NULL, 0);

	//Grundinizialisierung der Netzwerkkarte
	Init_Realtek_Network_Card();

	printf ("Card Init: %d \n",ReadRTL(0));
	//Ausgabe meiner MAC Adresse auf Serieller Schnittstelle
	printf ("\nMy Mac: %x.%x.%x.%x.%x.%x\n",mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);

	//Ausgabe meiner IP Adresse auf Serieller Schnittstelle
	printf ("My IP : %d.%d.%d.%d\n\n",MYIP[0],MYIP[1],MYIP[2],MYIP[3]);	

	//Globale Interrupts Einschalten
	sei ();
	
	//Interrupt 4 Mega103/128 Interrupt 0 bei Mega32 soll auslsen bei ansteigender Flanke

	#if defined (__AVR_ATmega32__)	
		MCUCR=0x01;
	#endif	
	
	#if defined (__AVR_ATmega103__)
		EICR = 0x03;
	#endif
	
	#if defined (__AVR_ATmega128__)
		EICR = 0x03;
	#endif
	
	//Setze Portb fr die Ausgnge auf output 
	DDRB = OUTPUT;

	//Interrupt 4 oder 0 Einschalten (fr die Netzwerkkarte)
	#if defined (__AVR_ATmega103__)
		enable_external_int (1<<INT4);
	#endif
	
	#if defined (__AVR_ATmega128__)
		enable_external_int (1<<INT4);
	#endif
	
	#if defined (__AVR_ATmega32__)	
		enable_external_int (1<<INT0);
	#endif	

	#if USE_CLOCK
	//Interrupt for the Clock enable
	timer_enable_int(_BV(TOIE1));
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); //Prescaler = 1024 
	
	#endif

	//startet die Network Card
	WriteRTL (CR , 0x22);

	//Hauptprogramm luft stndig in einer schleife und macht nichts
	while (1)
		{
		}
return (1);
}

